@@ -40,22 +40,22 @@
  * 0008: data payload
  */
 
-static char data_hdr[8] = {
+static const char data_hdr[8] = {
 	'S', 'T', 'F', 1,
 	0, 0,		/* Length */
 	AC_PKT_DATA,	/* Type */
 	0		/* Unknown */
 };
 
-static struct pkt keepalive_pkt = {
+static const struct pkt keepalive_pkt = {
 	.hdr = { 'S', 'T', 'F', 1, 0, 0, AC_PKT_KEEPALIVE, 0 },
 };
 
-static struct pkt dpd_pkt = {
+static const struct pkt dpd_pkt = {
 	.hdr = { 'S', 'T', 'F', 1, 0, 0, AC_PKT_DPD_OUT, 0 },
 };
 
-static struct pkt dpd_resp_pkt = {
+static const struct pkt dpd_resp_pkt = {
 	.hdr = { 'S', 'T', 'F', 1, 0, 0, AC_PKT_DPD_RESP, 0 },
 };
 
@@ -188,8 +188,15 @@ static int start_cstp_connection(struct openconnect_info *vpninfo)
 	buf_append(reqbuf, "Cookie: webvpn=%s\r\n", vpninfo->cookie);
 	buf_append(reqbuf, "X-CSTP-Version: 1\r\n");
 	buf_append(reqbuf, "X-CSTP-Hostname: %s\r\n", vpninfo->localname);
-	if (vpninfo->deflate && i < sizeof(buf))
-		buf_append(reqbuf, "X-CSTP-Accept-Encoding: deflate;q=1.0\r\n");
+	if (vpninfo->req_compr) {
+		char sep = ' ';
+		buf_append(reqbuf, "X-CSTP-Accept-Encoding:");
+		if (vpninfo->req_compr & COMPR_DEFLATE) {
+			buf_append(reqbuf, "%cdeflate", sep);
+			sep = ',';
+		}
+		buf_append(reqbuf, "\r\n");
+	}
 	if (base_mtu)
 		buf_append(reqbuf, "X-CSTP-Base-MTU: %d\r\n", base_mtu);
 	buf_append(reqbuf, "X-CSTP-MTU: %d\r\n", mtu);
@@ -267,7 +274,7 @@ static int start_cstp_connection(struct openconnect_info *vpninfo)
 	vpn_progress(vpninfo, PRG_INFO, _("Got CONNECT response: %s\n"), buf);
 
 	/* We may have advertised it, but we only do it if the server agrees */
-	vpninfo->deflate = 0;
+	vpninfo->cstp_compr = vpninfo->dtls_compr = 0;
 	mtu = 0;
 
 	while ((i = vpninfo->ssl_gets(vpninfo, buf, sizeof(buf)))) {
@@ -367,7 +374,7 @@ static int start_cstp_connection(struct openconnect_info *vpninfo)
 				vpninfo->ssl_times.rekey_method = REKEY_NONE;
 		} else if (!strcmp(buf + 7, "Content-Encoding")) {
 			if (!strcmp(colon, "deflate"))
-				vpninfo->deflate = 1;
+				vpninfo->cstp_compr = COMPR_DEFLATE;
 			else {
 				vpn_progress(vpninfo, PRG_ERR,
 					     _("Unknown CSTP-Content-Encoding %s\n"),
@@ -531,6 +538,7 @@ static int start_cstp_connection(struct openconnect_info *vpninfo)
 int openconnect_make_cstp_connection(struct openconnect_info *vpninfo)
 {
 	int ret;
+	int deflate_bufsize = 0;
 
 	/* This needs to be done before openconnect_setup_dtls() because it's
 	   sent with the CSTP CONNECT handshake. Even if we don't end up doing
@@ -546,7 +554,13 @@ int openconnect_make_cstp_connection(struct openconnect_info *vpninfo)
 	if (ret)
 		return ret;
 
-	if (vpninfo->deflate) {
+	ret = start_cstp_connection(vpninfo);
+	if (ret)
+		goto out;
+
+	/* If deflate compression is enabled (which is CSTP-only), it needs its
+	 * context to be allocated. */
+	if (vpninfo->cstp_compr == COMPR_DEFLATE) {
 		vpninfo->deflate_adler32 = 1;
 		vpninfo->inflate_adler32 = 1;
 
@@ -554,28 +568,37 @@ int openconnect_make_cstp_connection(struct openconnect_info *vpninfo)
 		    deflateInit2(&vpninfo->deflate_strm, Z_DEFAULT_COMPRESSION,
 				 Z_DEFLATED, -12, 9, Z_DEFAULT_STRATEGY)) {
 			vpn_progress(vpninfo, PRG_ERR, _("Compression setup failed\n"));
-			vpninfo->deflate = 0;
+			ret = -ENOMEM;
+			goto out;
 		}
 
-		if (!vpninfo->deflate_pkt) {
-			vpninfo->deflate_pkt = malloc(sizeof(struct pkt) + 2048);
-			if (!vpninfo->deflate_pkt) {
-				vpn_progress(vpninfo, PRG_ERR,
-					     _("Allocation of deflate buffer failed\n"));
-				inflateEnd(&vpninfo->inflate_strm);
-				deflateEnd(&vpninfo->deflate_strm);
-				vpninfo->deflate = 0;
-			} else {
-				memset(vpninfo->deflate_pkt, 0, sizeof(struct pkt));
-				memcpy(vpninfo->deflate_pkt->hdr, data_hdr, 8);
-				vpninfo->deflate_pkt->hdr[6] = AC_PKT_COMPRESSED;
-			}
-		}
+		/* Add four bytes for the adler32 */
+		deflate_bufsize = deflateBound(&vpninfo->deflate_strm,
+					       vpninfo->ip_info.mtu) + 4;
 	}
 
-	ret = start_cstp_connection(vpninfo);
+	/* If *any* compression is enabled, we'll need a deflate_pkt to compress into */
+	if (deflate_bufsize > vpninfo->deflate_pkt_size) {
+		free(vpninfo->deflate_pkt);
+		vpninfo->deflate_pkt = malloc(sizeof(struct pkt) + deflate_bufsize);
+		if (!vpninfo->deflate_pkt) {
+			vpninfo->deflate_pkt_size = 0;
+			vpn_progress(vpninfo, PRG_ERR,
+				     _("Allocation of deflate buffer failed\n"));
+			ret = -ENOMEM;
+			goto out;
+		}
+
+		vpninfo->deflate_pkt_size = deflate_bufsize;
+		memset(vpninfo->deflate_pkt, 0, sizeof(struct pkt));
+		memcpy(vpninfo->deflate_pkt->hdr, data_hdr, 8);
+		vpninfo->deflate_pkt->hdr[6] = AC_PKT_COMPRESSED;
+	}
+
+ out:
 	if (ret < 0)
 		openconnect_close_https(vpninfo, 0);
+
 	return ret;
 }
 
@@ -587,7 +610,7 @@ static int cstp_reconnect(struct openconnect_info *vpninfo)
 
 	openconnect_close_https(vpninfo, 0);
 
-	if (vpninfo->deflate) {
+	if (vpninfo->cstp_compr == COMPR_DEFLATE) {
 		/* Requeue the original packet that was deflated */
 		if (vpninfo->current_ssl_pkt == vpninfo->deflate_pkt) {
 			vpninfo->current_ssl_pkt = NULL;
@@ -600,6 +623,11 @@ static int cstp_reconnect(struct openconnect_info *vpninfo)
 	timeout = vpninfo->reconnect_timeout;
 	interval = vpninfo->reconnect_interval;
 
+	free(vpninfo->dtls_pkt);
+	vpninfo->dtls_pkt = NULL;
+	free(vpninfo->tun_pkt);
+	vpninfo->tun_pkt = NULL;
+
 	while ((ret = openconnect_make_cstp_connection(vpninfo))) {
 		if (timeout <= 0)
 			return ret;
@@ -752,8 +780,7 @@ static int cstp_write(struct openconnect_info *vpninfo, void *buf, int buflen)
 
 int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 {
-	unsigned char buf[16384];
-	int len, ret;
+	int ret;
 	int work_done = 0;
 
 	if (vpninfo->ssl_fd == -1)
@@ -765,26 +792,49 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 	   we should probably remove POLLIN from the events we're looking for,
 	   and add POLLOUT. As it is, though, it'll just chew CPU time in that
 	   fairly unlikely situation, until the write backlog clears. */
-	while ((len = cstp_read(vpninfo, buf, sizeof(buf))) > 0) {
+	while (1) {
+		int len = vpninfo->deflate_pkt_size ? : vpninfo->ip_info.mtu;
 		int payload_len;
 
-		if (buf[0] != 'S' || buf[1] != 'T' ||
-		    buf[2] != 'F' || buf[3] != 1 || buf[7])
+		if (!vpninfo->cstp_pkt) {
+			vpninfo->cstp_pkt = malloc(sizeof(struct pkt) + len);
+			if (!vpninfo->cstp_pkt) {
+				vpn_progress(vpninfo, PRG_ERR, _("Allocation failed\n"));
+				break;
+			}
+		}
+
+		len = cstp_read(vpninfo, vpninfo->cstp_pkt->hdr, len + 8);
+		if (!len)
+			break;
+		if (len < 0)
+			goto do_reconnect;
+		if (len < 8) {
+			vpn_progress(vpninfo, PRG_ERR, _("Short packet received (%d bytes)\n"), len);
+			vpninfo->quit_reason = "Short packet received";
+			return 1;
+		}
+
+		if (vpninfo->cstp_pkt->hdr[0] != 'S' || vpninfo->cstp_pkt->hdr[1] != 'T' ||
+		    vpninfo->cstp_pkt->hdr[2] != 'F' || vpninfo->cstp_pkt->hdr[3] != 1 ||
+		    vpninfo->cstp_pkt->hdr[7])
 			goto unknown_pkt;
 
-		payload_len = (buf[4] << 8) + buf[5];
+		payload_len = (vpninfo->cstp_pkt->hdr[4] << 8) + vpninfo->cstp_pkt->hdr[5];
 		if (len != 8 + payload_len) {
 			vpn_progress(vpninfo, PRG_ERR,
 				     _("Unexpected packet length. SSL_read returned %d but packet is\n"),
 				     len);
 			vpn_progress(vpninfo, PRG_ERR,
 				     "%02x %02x %02x %02x %02x %02x %02x %02x\n",
-				     buf[0], buf[1], buf[2], buf[3],
-				     buf[4], buf[5], buf[6], buf[7]);
+				     vpninfo->cstp_pkt->hdr[0], vpninfo->cstp_pkt->hdr[1],
+				     vpninfo->cstp_pkt->hdr[2], vpninfo->cstp_pkt->hdr[3],
+				     vpninfo->cstp_pkt->hdr[4], vpninfo->cstp_pkt->hdr[5],
+				     vpninfo->cstp_pkt->hdr[6], vpninfo->cstp_pkt->hdr[7]);
 			continue;
 		}
 		vpninfo->ssl_times.last_rx = time(NULL);
-		switch (buf[6]) {
+		switch (vpninfo->cstp_pkt->hdr[6]) {
 		case AC_PKT_DPD_OUT:
 			vpn_progress(vpninfo, PRG_DEBUG,
 				     _("Got CSTP DPD request\n"));
@@ -805,31 +855,32 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 			vpn_progress(vpninfo, PRG_TRACE,
 				     _("Received uncompressed data packet of %d bytes\n"),
 				     payload_len);
-			queue_new_packet(&vpninfo->incoming_queue, buf + 8,
-					 payload_len);
+			vpninfo->cstp_pkt->len = payload_len;
+			queue_packet(&vpninfo->incoming_queue, vpninfo->cstp_pkt);
+			vpninfo->cstp_pkt = NULL;
 			work_done = 1;
 			continue;
 
 		case AC_PKT_DISCONN: {
 			int i;
-			for (i = 0; i < payload_len; i++) {
-				if (!isprint(buf[payload_len + 8 + i]))
-					buf[payload_len + 8 + i] = '.';
+			for (i = 1; i < payload_len; i++) {
+				if (!isprint(vpninfo->cstp_pkt->data[i]))
+					vpninfo->cstp_pkt->data[i] = '.';
 			}
-			buf[payload_len + 8] = 0;
+			vpninfo->cstp_pkt->data[payload_len] = 0;
 			vpn_progress(vpninfo, PRG_ERR,
 				     _("Received server disconnect: %02x '%s'\n"),
-				     buf[8], buf + 9);
+				     vpninfo->cstp_pkt->data[0], vpninfo->cstp_pkt->data + 1);
 			vpninfo->quit_reason = "Server request";
 			return -EPIPE;
 		}
 		case AC_PKT_COMPRESSED:
-			if (!vpninfo->deflate) {
+			if (!vpninfo->cstp_compr) {
 				vpn_progress(vpninfo, PRG_ERR,
 					     _("Compressed packet received in !deflate mode\n"));
 				goto unknown_pkt;
 			}
-			inflate_and_queue_packet(vpninfo, buf + 8, payload_len);
+			inflate_and_queue_packet(vpninfo, vpninfo->cstp_pkt->data, payload_len);
 			work_done = 1;
 			continue;
 
@@ -842,13 +893,13 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 	unknown_pkt:
 		vpn_progress(vpninfo, PRG_ERR,
 			     _("Unknown packet %02x %02x %02x %02x %02x %02x %02x %02x\n"),
-			     buf[0], buf[1], buf[2], buf[3],
-			     buf[4], buf[5], buf[6], buf[7]);
+			     vpninfo->cstp_pkt->hdr[0], vpninfo->cstp_pkt->hdr[1],
+			     vpninfo->cstp_pkt->hdr[2], vpninfo->cstp_pkt->hdr[3],
+			     vpninfo->cstp_pkt->hdr[4], vpninfo->cstp_pkt->hdr[5],
+			     vpninfo->cstp_pkt->hdr[6], vpninfo->cstp_pkt->hdr[7]);
 		vpninfo->quit_reason = "Unknown packet received";
 		return 1;
 	}
-	if (len < 0)
-		goto do_reconnect;
 
 
 	/* If SSL_write() fails we are expected to try again. With exactly
@@ -902,7 +953,7 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 
 	if (vpninfo->owe_ssl_dpd_response) {
 		vpninfo->owe_ssl_dpd_response = 0;
-		vpninfo->current_ssl_pkt = &dpd_resp_pkt;
+		vpninfo->current_ssl_pkt = (struct pkt *)&dpd_resp_pkt;
 		goto handle_outgoing;
 	}
 
@@ -951,7 +1002,7 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 	case KA_DPD:
 		vpn_progress(vpninfo, PRG_DEBUG, _("Send CSTP DPD\n"));
 
-		vpninfo->current_ssl_pkt = &dpd_pkt;
+		vpninfo->current_ssl_pkt = (struct pkt *)&dpd_pkt;
 		goto handle_outgoing;
 
 	case KA_KEEPALIVE:
@@ -962,7 +1013,7 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 
 		vpn_progress(vpninfo, PRG_DEBUG, _("Send CSTP Keepalive\n"));
 
-		vpninfo->current_ssl_pkt = &keepalive_pkt;
+		vpninfo->current_ssl_pkt = (struct pkt *)&keepalive_pkt;
 		goto handle_outgoing;
 
 	case KA_NONE:
@@ -975,14 +1026,14 @@ int cstp_mainloop(struct openconnect_info *vpninfo, int *timeout)
 		vpninfo->outgoing_queue = this->next;
 		vpninfo->outgoing_qlen--;
 
-		if (vpninfo->deflate) {
+		if (vpninfo->cstp_compr == COMPR_DEFLATE) {
 			unsigned char *adler;
 			int ret;
 
 			vpninfo->deflate_strm.next_in = this->data;
 			vpninfo->deflate_strm.avail_in = this->len;
 			vpninfo->deflate_strm.next_out = (void *)vpninfo->deflate_pkt->data;
-			vpninfo->deflate_strm.avail_out = 2040;
+			vpninfo->deflate_strm.avail_out = vpninfo->deflate_pkt_size - 4;
 			vpninfo->deflate_strm.total_out = 0;
 
 			ret = deflate(&vpninfo->deflate_strm, Z_SYNC_FLUSH);
