@@ -72,7 +72,7 @@ struct openconnect_info *openconnect_vpninfo_new(const char *useragent,
 	vpninfo->ssl_fd = vpninfo->dtls_fd = -1;
 	vpninfo->cmd_fd = vpninfo->cmd_fd_write = -1;
 	vpninfo->cert_expire_warning = 60 * 86400;
-	vpninfo->deflate = 1;
+	vpninfo->req_compr = COMPR_ALL;
 	vpninfo->max_qlen = 10;
 	vpninfo->localname = strdup("localhost");
 	vpninfo->useragent = openconnect_create_useragent(useragent);
@@ -238,6 +238,16 @@ void openconnect_vpninfo_free(struct openconnect_info *vpninfo)
 #endif
 		vpninfo->peer_cert = NULL;
 	}
+	while (vpninfo->pin_cache) {
+		struct pin_cache *cache = vpninfo->pin_cache;
+
+		free(cache->token);
+		memset(cache->pin, 0x5a, strlen(cache->pin));
+		free(cache->pin);
+		vpninfo->pin_cache = cache->next;
+		free(cache);
+	}
+
 	free(vpninfo->peer_cert_hash);
 	free(vpninfo->localname);
 	free(vpninfo->useragent);
@@ -266,6 +276,17 @@ void openconnect_vpninfo_free(struct openconnect_info *vpninfo)
 	}
 	memset(vpninfo->yubikey_pwhash, 0, sizeof(vpninfo->yubikey_pwhash));
 	free(vpninfo->yubikey_objname);
+#endif
+#ifdef HAVE_LIBP11
+	if (vpninfo->pkcs11_ctx) {
+		if (vpninfo->pkcs11_slot_list)
+			PKCS11_release_all_slots(vpninfo->pkcs11_ctx,
+						 vpninfo->pkcs11_slot_list,
+						 vpninfo->pkcs11_slot_count);
+		PKCS11_CTX_unload(vpninfo->pkcs11_ctx);
+		PKCS11_CTX_free(vpninfo->pkcs11_ctx);
+	}
+	free(vpninfo->pkcs11_cert_id);
 #endif
 	/* These check strm->state so they are safe to call multiple times */
 	inflateEnd(&vpninfo->inflate_strm);
@@ -273,6 +294,8 @@ void openconnect_vpninfo_free(struct openconnect_info *vpninfo)
 
 	free(vpninfo->deflate_pkt);
 	free(vpninfo->tun_pkt);
+	free(vpninfo->dtls_pkt);
+	free(vpninfo->cstp_pkt);
 	free(vpninfo);
 }
 
@@ -509,6 +532,8 @@ int openconnect_has_pkcs11_support(void)
 {
 #if defined(OPENCONNECT_GNUTLS) && defined(HAVE_P11KIT)
 	return 1;
+#elif defined(OPENCONNECT_OPENSSL) && defined(HAVE_LIBP11)
+	return 1;
 #else
 	return 0;
 #endif
