@@ -157,6 +157,7 @@ enum {
 	OPT_DTLS_CIPHERS,
 	OPT_DUMP_HTTP,
 	OPT_FORCE_DPD,
+	OPT_GNUTLS_DEBUG,
 	OPT_KEY_PASSWORD_FROM_FSID,
 	OPT_LIBPROXY,
 	OPT_NO_CERT_CHECK,
@@ -192,7 +193,7 @@ enum {
 #define OPTION(name, arg, abbrev) {name, arg, NULL, abbrev}
 #endif
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
 #ifndef _WIN32
 	OPTION("background", 0, 'b'),
 	OPTION("pid-file", 1, OPT_PIDFILE),
@@ -254,9 +255,19 @@ static struct option long_options[] = {
 	OPTION("no-xmlpost", 0, OPT_NO_XMLPOST),
 	OPTION("dump-http-traffic", 0, OPT_DUMP_HTTP),
 	OPTION("no-system-trust", 0, OPT_NO_SYSTEM_TRUST),
+#ifdef OPENCONNECT_GNUTLS
+	OPTION("gnutls-debug", 1, OPT_GNUTLS_DEBUG),
+#endif
 	OPTION(NULL, 0, 0)
 };
 
+#ifdef OPENCONNECT_GNUTLS
+static void oc_gnutls_log_func(int level, const char *str)
+{
+	fputs(str, stderr);
+}
+#endif
+
 #ifdef _WIN32
 static int __attribute__ ((format(printf, 2, 0)))
     vfprintf_utf8(FILE *f, const char *fmt, va_list args)
@@ -824,7 +835,7 @@ static int next_option(int argc, char **argv, char **config_arg)
 
 	ssize_t llen;
 	int opt, optlen = 0;
-	struct option *this;
+	const struct option *this;
 	char *line;
 	int ate_equals = 0;
 
@@ -1121,10 +1132,10 @@ int main(int argc, char **argv)
 			vpninfo->sslkey = dup_config_arg();
 			break;
 		case 'd':
-			vpninfo->deflate = 1;
+			vpninfo->req_compr = COMPR_ALL;
 			break;
 		case 'D':
-			vpninfo->deflate = 0;
+			vpninfo->req_compr = 0;
 			break;
 		case 'g':
 			free(urlpath);
@@ -1263,6 +1274,12 @@ int main(int argc, char **argv)
 		case OPT_TIMESTAMP:
 			timestamp = 1;
 			break;
+#ifdef OPENCONNECT_GNUTLS
+		case OPT_GNUTLS_DEBUG:
+			gnutls_global_set_log_level(atoi(config_arg));
+			gnutls_global_set_log_function(oc_gnutls_log_func);
+			break;
+#endif
 		default:
 			usage();
 		}
@@ -1412,7 +1429,7 @@ int main(int argc, char **argv)
 		     (ip_info->netmask6 && ip_info->addr) ? " + " : "",
 		     ip_info->netmask6 ? : "",
 		     (vpninfo->dtls_state != DTLS_CONNECTED) ?
-		     (vpninfo->deflate ? "SSL + deflate" : "SSL")
+		     (vpninfo->cstp_compr == COMPR_DEFLATE) ? "SSL + deflate" : "SSL"
 		     : "DTLS");
 
 	if (!vpninfo->vpnc_script) {
