@@ -89,6 +89,10 @@
 #include <pskc/pskc.h>
 #endif
 
+#ifdef HAVE_LIBP11
+#include <libp11.h>
+#endif
+
 #ifdef HAVE_LIBPCSCLITE
 #ifdef __APPLE__
 #include <PCSC/wintypes.h>
@@ -136,6 +140,9 @@ struct pkt {
 #define DTLS_CONNECTING	3
 #define DTLS_CONNECTED	4
 
+#define COMPR_DEFLATE	(1<<0)
+#define COMPR_ALL	(COMPR_DEFLATE)
+
 struct keepalive_info {
 	int dpd;
 	int keepalive;
@@ -310,6 +317,14 @@ struct openconnect_info {
 
 	unsigned pfs;
 #if defined(OPENCONNECT_OPENSSL)
+#ifdef HAVE_LIBP11
+	PKCS11_CTX *pkcs11_ctx;
+	PKCS11_SLOT *pkcs11_slot_list;
+	unsigned int pkcs11_slot_count;
+	PKCS11_SLOT *pkcs11_cert_slot;
+	unsigned char *pkcs11_cert_id;
+	size_t pkcs11_cert_id_len;
+ #endif
 	X509 *cert_x509;
 	SSL_CTX *https_ctx;
 	SSL *https_ssl;
@@ -317,7 +332,6 @@ struct openconnect_info {
 	gnutls_session_t https_sess;
 	gnutls_certificate_credentials_t https_cred;
 	char local_cert_md5[MD5_SIZE * 2 + 1]; /* For CSD */
-	struct pin_cache *pin_cache;
 #ifdef HAVE_TROUSERS
 	TSS_HCONTEXT tpm_context;
 	TSS_HKEY srk;
@@ -335,14 +349,20 @@ struct openconnect_info {
 	unsigned int nr_my_certs;
 #endif
 #endif /* OPENCONNECT_GNUTLS */
+	struct pin_cache *pin_cache;
 	struct keepalive_info ssl_times;
 	int owe_ssl_dpd_response;
-	struct pkt *deflate_pkt;
-	struct pkt *current_ssl_pkt;
-	struct pkt *pending_deflated_pkt;
 
+	int deflate_pkt_size;			/* It may need to be larger than MTU */
+	struct pkt *deflate_pkt;		/* For compressing outbound packets into */
+	struct pkt *pending_deflated_pkt;	/* The original packet associated with above */
+	struct pkt *current_ssl_pkt;		/* Partially sent SSL packet */
+
+	/* Packet buffers for receiving into */
+	struct pkt *cstp_pkt;
+	struct pkt *dtls_pkt;
 	struct pkt *tun_pkt;
-
+	
 	z_stream inflate_strm;
 	uint32_t inflate_adler32;
 	z_stream deflate_strm;
@@ -426,7 +446,10 @@ struct openconnect_info {
 
 	int dtls_local_port;
 
-	int deflate;
+	int req_compr; /* What we requested */
+	int cstp_compr; /* Accepted for CSTP */
+	int dtls_compr; /* Accepted for DTLS */
+
 	int is_dyndns; /* Attempt to redo DNS lookup on each CSTP reconnect */
 	char *useragent;
 
@@ -487,7 +510,7 @@ struct openconnect_info {
 #define AC_PKT_TERM_SERVER	9	/* Server kick */
 
 #define vpn_progress(vpninfo, ...) (vpninfo)->progress((vpninfo)->cbdata, __VA_ARGS__)
-#define vpn_perror(vpninfo, msg) vpn_progress((vpninfo), PRG_ERR, "%s: %s", (msg), strerror(errno));
+#define vpn_perror(vpninfo, msg) vpn_progress((vpninfo), PRG_ERR, "%s: %s\n", (msg), strerror(errno));
 
 /****************************************************************************/
 /* Oh Solaris how we hate thee! */
@@ -636,6 +659,10 @@ FILE *openconnect_fopen_utf8(struct openconnect_info *vpninfo,
 
 void openconnect_clear_cookies(struct openconnect_info *vpninfo);
 
+/* openssl-pkcs11.c */
+int load_pkcs11_key(struct openconnect_info *vpninfo);
+int load_pkcs11_certificate(struct openconnect_info *vpninfo);
+
 /* {gnutls,openssl}.c */
 int openconnect_open_https(struct openconnect_info *vpninfo);
 void openconnect_close_https(struct openconnect_info *vpninfo, int final);
