@@ -28,10 +28,30 @@
 #define uid_t unsigned
 #endif
 
-#define OPENCONNECT_API_VERSION_MAJOR 3
-#define OPENCONNECT_API_VERSION_MINOR 3
+#define OPENCONNECT_API_VERSION_MAJOR 5
+#define OPENCONNECT_API_VERSION_MINOR 0
 
 /*
+ * API version 5.0:
+ *  - Remove OPENCONNECT_X509 and openconnect_get_peer_cert().
+ *  - Change openconnect_get_cert_der() to openconnect_get_peer_cert_DER() etc.
+ *  - Add openconnect_check_peer_cert_hash().
+ *  - Remove openconnect_set_server_cert_sha1().
+ *  - Add openconnect_has_yubioath_support() and OC_TOKEN_MODE_YUBIOATH.
+ *  - Add openconnect_has_system_key_support().
+ *
+ * API version 4.1:
+ *  - Add openconnect_get_cstp_cipher(), openconnect_get_dtls_cipher(),
+ *    openconnect_set_system_trust(), openconnect_set_csd_environ().
+ *  - Change openconnect_init_ssl() to return int.
+ *
+ * API version 4.0:
+ *  - Change string handling to never transfer ownership of allocations.
+ *  - Add openconnect_set_option_value(), openconnect_free_cert_info().
+ *
+ * API version 3.4:
+ *  - Add openconnect_set_token_callbacks()
+ *
  * API version 3.3:
  *  - Add openconnect_set_pfs(), openconnect_set_dpd(),
  *    openconnect_set_proxy_auth()
@@ -48,7 +68,7 @@
  *    openconnect_get_ifname(), openconnect_set_reqmtu(),
  *    openconnect_get_ip_info(), openconnect_set_protect_socket_handler(),
  *    openconnect_set_mobile_info(), openconnect_set_xmlpost(),
- *    openconnect_set_stats_handler()
+7 *    openconnect_set_stats_handler()
  *
  * API version 3.0:
  *  - Change oc_form_opt_select->choices to an array of pointers
@@ -138,19 +158,22 @@
 #define OC_FORM_OPT_NUMERIC		0x0002
 
 /* char * fields are static (owned by XML parser) and don't need to be
-   freed by the form handling code -- except for value, which for TEXT
-   and PASSWORD options is allocated by process_form() when
-   interacting with the user and must be freed. */
+   freed by the form handling code — except for value, which for TEXT
+   and PASSWORD options is allocated by openconnect_set_option_value()
+   when process_form() interacts with the user and must be freed. */
 struct oc_form_opt {
 	struct oc_form_opt *next;
 	int type;
 	char *name;
 	char *label;
-	char *value;
+	char *_value; /* Use openconnect_set_option_value() to set this */
 	unsigned int flags;
 	void *reserved;
 };
 
+/* To set the value to a form use the following function */
+int openconnect_set_option_value(struct oc_form_opt *opt, const char *value);
+
 /* All fields are static, owned by the XML parser */
 struct oc_choice {
 	char *name;
@@ -248,59 +271,129 @@ struct oc_stats {
 
 struct openconnect_info;
 
-#define OPENCONNECT_X509 void
-
 typedef enum {
 	OC_TOKEN_MODE_NONE,
 	OC_TOKEN_MODE_STOKEN,
 	OC_TOKEN_MODE_TOTP,
 	OC_TOKEN_MODE_HOTP,
+	OC_TOKEN_MODE_YUBIOATH,
 } oc_token_mode_t;
 
-/* Unless otherwise specified, all functions which set strings will take
-   ownership of those strings and the library will free them later in
-   openconnect_vpninfo_free() */
+/* All strings are UTF-8. If operating in a legacy environment where
+   nl_langinfo(CODESET) returns anything other than UTF-8, or on Windows,
+   the library will take appropriate steps to convert back to the legacy
+   character set (or UTF-16) for file handling and wherever else it is
+   appropriate to do so. Library functions may (but probably don't yet)
+   return -EILSEQ if passed invalid UTF-8 strings. */
+
+/* Unlike previous versions of openconnect, no functions will take ownership
+   of the provided strings. */
 
 
-/* The buffer 'buf' must be at least 41 bytes. It will receive a hex string
-   with trailing NUL, representing the SHA1 fingerprint of the certificate. */
-int openconnect_get_cert_sha1(struct openconnect_info *vpninfo,
-			      OPENCONNECT_X509 *cert, char *buf);
-char *openconnect_get_cert_details(struct openconnect_info *vpninfo,
-				   OPENCONNECT_X509 *cert);
+/* Provide environment variables to be set in the CSD trojan environment
+   before spawning it. Some callers may need to set $TMPDIR, $PATH and
+   other such things if not running from a standard UNIX-like environment.
+   To ensure that a variable is unset, pass its name with value==NULL.
+   To clear all settings and allow the CSD trojan to inherit an unmodified
+   environment, call with name==NULL. */
+
+int openconnect_set_csd_environ(struct openconnect_info *vpninfo,
+				const char *name, const char *value);
+
+/* This string is static, valid only while the connection lasts. If you
+ * are going to cache this to remember which certs the user has accepted,
+ * make sure you also store the host/port for which it was accepted and
+ * don't just accept this cert from *anywhere*. Also use use the check
+ * function below instead of manually comparing. When this function
+ * returns a string which *doesn't* match the previously-stored hash
+ * matched with openconnect_check_peer_cert_hash(), you should store
+ * the new result from this function in place of the old. It means
+ * we have upgraded to a better hash function. */
+const char *openconnect_get_peer_cert_hash(struct openconnect_info *vpninfo);
+
+/* Check if the current peer certificate matches a hash previously
+ * obtained from openconect_get_peer_cert_hash(). Clients should not
+ * attempt to do this using strcmp() and the *current* result of
+ * openconnect_get_peer_cert_hash() because it might use
+ * a different hash function today. This function will get it right.
+ * Returns 0 on match; 1 on mismatch, -errno on failure. */
+int openconnect_check_peer_cert_hash(struct openconnect_info *vpninfo,
+				     const char *old_hash);
+
+/* The buffers returned by these two functions must be freed with
+   openconnect_free_cert_info(), especially on Windows. */
+char *openconnect_get_peer_cert_details(struct openconnect_info *vpninfo);
+
 /* Returns the length of the created DER output, in a newly-allocated buffer
-   that will need to be freed by the caller. */
-int openconnect_get_cert_DER(struct openconnect_info *vpninfo,
-			     OPENCONNECT_X509 *cert, unsigned char **buf);
-
+   that will need to be freed by openconnect_free_cert_info(). */
+int openconnect_get_peer_cert_DER(struct openconnect_info *vpninfo,
+				  unsigned char **buf);
+void openconnect_free_cert_info(struct openconnect_info *vpninfo,
+				void *buf);
 /* Contains a comma-separated list of authentication methods to enabled.
    Currently supported: Negotiate,NTLM,Digest,Basic */
-int openconnect_set_proxy_auth(struct openconnect_info *vpninfo, char *methods);
-int openconnect_set_http_proxy(struct openconnect_info *vpninfo, char *proxy);
+int openconnect_set_proxy_auth(struct openconnect_info *vpninfo,
+			       const char *methods);
+int openconnect_set_http_proxy(struct openconnect_info *vpninfo,
+			       const char *proxy);
 int openconnect_passphrase_from_fsid(struct openconnect_info *vpninfo);
 int openconnect_obtain_cookie(struct openconnect_info *vpninfo);
-void openconnect_init_ssl(void);
+int openconnect_init_ssl(void);
 
-char *openconnect_get_hostname(struct openconnect_info *);
-void openconnect_set_hostname(struct openconnect_info *, char *);
+/* These are strictly cosmetic. The strings differ depending on
+ * whether OpenSSL or GnuTLS is being used. And even depending on the
+ * version of GnuTLS. Do *not* attempt to do anything meaningful based
+ * on matching these strings; if you want to do something like that then
+ * ask for an API that *does* offer you what you need. */
+const char *openconnect_get_cstp_cipher(struct openconnect_info *);
+const char *openconnect_get_dtls_cipher(struct openconnect_info *);
+
+const char *openconnect_get_hostname(struct openconnect_info *);
+int openconnect_set_hostname(struct openconnect_info *, const char *);
 char *openconnect_get_urlpath(struct openconnect_info *);
-void openconnect_set_urlpath(struct openconnect_info *, char *);
+int openconnect_set_urlpath(struct openconnect_info *, const char *);
+
+/* Some software tokens, such as HOTP tokens, include a counter which
+ * needs to be stored in persistent storage.
+ *
+ * For such tokens, the lock function is first invoked to obtain a lock
+ * on the storage because we're about to generate a new code. It is
+ * permitted to call openconnect_set_token_mode() from the lock function,
+ * if the token storage has been updated since it was first loaded. The
+ * token mode must not change; only the token secret.
+ *
+ * The unlock function is called when a token code has been generated,
+ * with a new token secret to be written to the persistent storage. The
+ * secret will be in the same format as it was originally received by
+ * openconnect_set_token_mode(). The new token may be NULL if an error
+ * was encountered generating the code, in which case it is only
+ * necessary for the callback function to unlock the storage.
+ */
+typedef int (*openconnect_lock_token_vfn)(void *tokdata);
+typedef int (*openconnect_unlock_token_vfn)(void *tokdata, const char *new_tok);
+int openconnect_set_token_callbacks(struct openconnect_info *, void *tokdata,
+				    openconnect_lock_token_vfn,
+				    openconnect_unlock_token_vfn);
 
-/* These functions do *not* take ownership of the string; it is parsed
-   and then discarded. */
 int openconnect_set_token_mode(struct openconnect_info *,
 			       oc_token_mode_t, const char *token_str);
 /* Legacy stoken-only function; do not use */
 int openconnect_set_stoken_mode(struct openconnect_info *, int, const char *);
 
-/* This function does *not* take ownership of the string; it's copied
-   into a static buffer in the vpninfo. The size must be 41 bytes,
-   since that's the size of a 20-byte SHA1 represented as hex with
-   a trailing NUL. */
+/* The size must be 41 bytes, since that's the size of a 20-byte SHA1
+   represented as hex with a trailing NUL. */
 void openconnect_set_xmlsha1(struct openconnect_info *, const char *, int size);
 
-void openconnect_set_cafile(struct openconnect_info *, char *);
-void openconnect_setup_csd(struct openconnect_info *, uid_t, int silent, char *wrapper);
+int openconnect_set_cafile(struct openconnect_info *, const char *);
+
+/* call this function to disable the system trust from being used to
+ * verify the server certificate. @val is a boolean value.
+ *
+ * For backwards compatibility reasons this is enabled by default.
+ */
+void openconnect_set_system_trust(struct openconnect_info *vpninfo, unsigned val);
+
+int openconnect_setup_csd(struct openconnect_info *, uid_t, int silent, const char *wrapper);
 void openconnect_set_xmlpost(struct openconnect_info *, int enable);
 
 /* Valid choices are: "linux", "linux-64", "win", "mac-intel",
@@ -308,12 +401,12 @@ void openconnect_set_xmlpost(struct openconnect_info *, int enable);
    trojan binary. */
 int openconnect_set_reported_os(struct openconnect_info *, const char *os);
 
-void openconnect_set_mobile_info(struct openconnect_info *vpninfo,
-				 char *mobile_platform_version,
-				 char *mobile_device_type,
-				 char *mobile_device_uniqueid);
-void openconnect_set_client_cert(struct openconnect_info *, char *cert, char *sslkey);
-void openconnect_set_server_cert_sha1(struct openconnect_info *, char *);
+int openconnect_set_mobile_info(struct openconnect_info *vpninfo,
+				const char *mobile_platform_version,
+				const char *mobile_device_type,
+				const char *mobile_device_uniqueid);
+int openconnect_set_client_cert(struct openconnect_info *, const char *cert,
+				const char *sslkey);
 const char *openconnect_get_ifname(struct openconnect_info *);
 void openconnect_set_reqmtu(struct openconnect_info *, int reqmtu);
 void openconnect_set_dpd(struct openconnect_info *, int min_seconds);
@@ -327,18 +420,12 @@ int openconnect_get_ip_info(struct openconnect_info *,
 			    const struct oc_vpn_option **cstp_options,
 			    const struct oc_vpn_option **dtls_options);
 
-/* This is *not* yours and must not be destroyed with X509_free(). It
-   will be valid when a cookie has been obtained successfully, and will
-   be valid until the connection is destroyed or another attempt it made
-   to use it. */
-OPENCONNECT_X509 *openconnect_get_peer_cert(struct openconnect_info *);
-
 int openconnect_get_port(struct openconnect_info *);
-char *openconnect_get_cookie(struct openconnect_info *);
+const char *openconnect_get_cookie(struct openconnect_info *);
 void openconnect_clear_cookie(struct openconnect_info *);
 
 void openconnect_reset_ssl(struct openconnect_info *vpninfo);
-int openconnect_parse_url(struct openconnect_info *vpninfo, char *url);
+int openconnect_parse_url(struct openconnect_info *vpninfo, const char *url);
 void openconnect_set_cert_expiry_warning(struct openconnect_info *vpninfo,
 					 int seconds);
 void openconnect_set_pfs(struct openconnect_info *vpninfo, unsigned val);
@@ -358,7 +445,12 @@ void openconnect_set_cancel_fd(struct openconnect_info *vpninfo, int fd);
    to the library. This returns a file descriptor to the write side of
    the pipe. Both sides will be closed by openconnect_vpninfo_free().
    This replaces openconnect_set_cancel_fd(). */
-int openconnect_setup_cmd_pipe(struct openconnect_info *vpninfo);
+#ifdef _WIN32
+SOCKET
+#else
+int
+#endif
+openconnect_setup_cmd_pipe(struct openconnect_info *vpninfo);
 
 const char *openconnect_get_version(void);
 
@@ -367,10 +459,12 @@ int openconnect_make_cstp_connection(struct openconnect_info *vpninfo);
 
 /* Create a tun device through the OS kernel (typical use case). Both
    strings are optional and can be NULL if desired. */
-int openconnect_setup_tun_device(struct openconnect_info *vpninfo, char *vpnc_script, char *ifname);
+int openconnect_setup_tun_device(struct openconnect_info *vpninfo,
+				 const char *vpnc_script, const char *ifname);
 
 /* Pass traffic to a script program (no tun device). */
-int openconnect_setup_tun_script(struct openconnect_info *vpninfo, char *tun_script);
+int openconnect_setup_tun_script(struct openconnect_info *vpninfo,
+				 const char *tun_script);
 
 #ifdef _WIN32
 /* Caller will provide an overlap-capable handle for the tunnel traffic. */
@@ -401,7 +495,6 @@ int openconnect_mainloop(struct openconnect_info *vpninfo,
    if the certificate is (or has in the past been) explicitly accepted
    by the user, and non-zero to abort the connection. */
 typedef int (*openconnect_validate_peer_cert_vfn) (void *privdata,
-						   OPENCONNECT_X509 *cert,
 						   const char *reason);
 /* On a successful connection, the server may provide us with a new XML
    configuration file. This contains the list of servers that can be
@@ -410,7 +503,7 @@ typedef int (*openconnect_validate_peer_cert_vfn) (void *privdata,
    not match the SHA1 set with the openconnect_set_xmlsha1() above. If
    they don't match, or openconnect_set_xmlsha1() has not been called,
    then the new XML is downloaded and this function is invoked. */
-typedef int (*openconnect_write_new_config_vfn) (void *privdata, char *buf,
+typedef int (*openconnect_write_new_config_vfn) (void *privdata, const char *buf,
 						int buflen);
 /* Handle an authentication form, requesting input from the user.
  * Return value:
@@ -424,7 +517,7 @@ typedef int (*openconnect_process_auth_form_vfn) (void *privdata,
 typedef void __attribute__ ((format(printf, 3, 4)))
 		(*openconnect_progress_vfn) (void *privdata, int level,
 					    const char *fmt, ...);
-struct openconnect_info *openconnect_vpninfo_new(char *useragent,
+struct openconnect_info *openconnect_vpninfo_new(const char *useragent,
 						 openconnect_validate_peer_cert_vfn,
 						 openconnect_write_new_config_vfn,
 						 openconnect_process_auth_form_vfn,
@@ -457,5 +550,7 @@ int openconnect_has_tss_blob_support(void);
 /* Software token capabilities. */
 int openconnect_has_stoken_support(void);
 int openconnect_has_oath_support(void);
+int openconnect_has_yubioath_support(void);
+int openconnect_has_system_key_support(void);
 
 #endif /* __OPENCONNECT_H__ */
