@@ -35,13 +35,10 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
-#if defined(__sun__)
-#include <stropts.h>
-#include <sys/sockio.h>
-#include <net/if_tun.h>
-#ifndef TUNNEWPPA
-#error "Install TAP driver from http://www.whiteboard.ne.jp/~admin2/tuntap/"
-#endif
+#if defined(__APPLE__) && defined(HAVE_NET_UTUN_H)
+#include <sys/kern_control.h>
+#include <sys/sys_domain.h>
+#include <net/if_utun.h>
 #endif
 
 #include "openconnect-internal.h"
@@ -72,43 +69,27 @@
 #define TUN_HAS_AF_PREFIX 1
 #endif
 
-static int set_tun_mtu(struct openconnect_info *vpninfo)
-{
-#ifndef __sun__ /* We don't know how to do this on Solaris */
-	struct ifreq ifr;
-	int net_fd;
-
-	net_fd = socket(PF_INET, SOCK_DGRAM, 0);
-	if (net_fd < 0) {
-		perror(_("open net"));
-		return -EINVAL;
-	}
-
-	memset(&ifr, 0, sizeof(ifr));
-	strncpy(ifr.ifr_name, vpninfo->ifname, sizeof(ifr.ifr_name) - 1);
-	ifr.ifr_mtu = vpninfo->ip_info.mtu;
-
-	if (ioctl(net_fd, SIOCSIFMTU, &ifr) < 0)
-		perror(_("SIOCSIFMTU"));
-
-	close(net_fd);
-#endif
-	return 0;
-}
-
 #ifdef __sun__
-static int link_proto(int unit_nr, const char *devname, uint64_t flags)
+#include <stropts.h>
+#include <sys/sockio.h>
+
+#ifndef TUNNEWPPA
+#error "Install TAP driver from http://www.whiteboard.ne.jp/~admin2/tuntap/"
+#endif
+
+static int link_proto(struct openconnect_info *vpninfo, int unit_nr,
+		      const char *devname, uint64_t flags)
 {
 	int ip_fd, mux_id, tun2_fd;
 	struct lifreq ifr;
 
 	tun2_fd = open("/dev/tun", O_RDWR);
 	if (tun2_fd < 0) {
-		perror(_("Could not open /dev/tun for plumbing"));
+		vpn_perror(vpninfo, _("Could not open /dev/tun for plumbing"));
 		return -EIO;
 	}
 	if (ioctl(tun2_fd, I_PUSH, "ip") < 0) {
-		perror(_("Can't push IP"));
+		vpn_perror(vpninfo, _("Can't push IP"));
 		close(tun2_fd);
 		return -EIO;
 	}
@@ -118,24 +99,25 @@ static int link_proto(int unit_nr, const char *devname, uint64_t flags)
 	ifr.lifr_flags = flags;
 
 	if (ioctl(tun2_fd, SIOCSLIFNAME, &ifr) < 0) {
-		perror(_("Can't set ifname"));
+		vpn_perror(vpninfo, _("Can't set ifname"));
 		close(tun2_fd);
 		return -1;
 	}
 
 	ip_fd = open(devname, O_RDWR);
 	if (ip_fd < 0) {
-		fprintf(stderr, _("Can't open %s: %s"), devname,
-			strerror(errno));
+		vpn_progress(vpninfo, PRG_ERR, _("Can't open %s: %s"),
+			     devname, strerror(errno));
 		close(tun2_fd);
 		return -1;
 	}
 
 	mux_id = ioctl(ip_fd, I_LINK, tun2_fd);
 	if (mux_id < 0) {
-		fprintf(stderr, _("Can't plumb %s for IPv%d: %s\n"),
-			 ifr.lifr_name, (flags == IFF_IPV4) ? 4 : 6,
-			 strerror(errno));
+		vpn_progress(vpninfo, PRG_ERR,
+			     _("Can't plumb %s for IPv%d: %s\n"),
+			     ifr.lifr_name, (flags == IFF_IPV4) ? 4 : 6,
+			     strerror(errno));
 		close(tun2_fd);
 		close(ip_fd);
 		return -1;
@@ -145,40 +127,91 @@ static int link_proto(int unit_nr, const char *devname, uint64_t flags)
 
 	return ip_fd;
 }
-#endif
-
-#ifdef SIOCIFCREATE
-static int bsd_open_tun(char *tun_name)
-{
-	int fd;
-	int s;
-	struct ifreq ifr;
-
-	fd = open(tun_name, O_RDWR);
-	if (fd >= 0) {
-		return fd;
-
-		s = socket(AF_INET, SOCK_DGRAM, 0);
-		if (s < 0)
-			return -1;
-
-		memset(&ifr, 0, sizeof(ifr));
-		strncpy(ifr.ifr_name, tun_name + 5, sizeof(ifr.ifr_name) - 1);
-		if (!ioctl(s, SIOCIFCREATE, &ifr))
-			fd = open(tun_name, O_RDWR);
-
-		close(s);
-	}
-	return fd;
-}
-#else
-#define bsd_open_tun(tun_name) open(tun_name, O_RDWR)
-#endif
 
 intptr_t os_setup_tun(struct openconnect_info *vpninfo)
 {
 	int tun_fd = -1;
+	static char tun_name[80];
+	int unit_nr;
+
+	tun_fd = open("/dev/tun", O_RDWR);
+	if (tun_fd < 0) {
+		vpn_perror(vpninfo, _("open /dev/tun"));
+		return -EIO;
+	}
+
+	unit_nr = ioctl(tun_fd, TUNNEWPPA, -1);
+	if (unit_nr < 0) {
+		vpn_perror(vpninfo, _("Failed to create new tun"));
+		close(tun_fd);
+		return -EIO;
+	}
+
+	if (ioctl(tun_fd, I_SRDOPT, RMSGD) < 0) {
+		vpn_perror(vpninfo, _("Failed to put tun file descriptor into message-discard mode"));
+		close(tun_fd);
+		return -EIO;
+	}
+
+	sprintf(tun_name, "tun%d", unit_nr);
+	vpninfo->ifname = strdup(tun_name);
+
+	vpninfo->ip_fd = link_proto(vpninfo, unit_nr, "/dev/udp", IFF_IPV4);
+	if (vpninfo->ip_fd < 0) {
+		close(tun_fd);
+		return -EIO;
+	}
+
+	if (vpninfo->ip_info.addr6) {
+		vpninfo->ip6_fd = link_proto(vpninfo, unit_nr, "/dev/udp6", IFF_IPV6);
+		if (vpninfo->ip6_fd < 0) {
+			close(tun_fd);
+			close(vpninfo->ip_fd);
+			vpninfo->ip_fd = -1;
+			return -EIO;
+		}
+	} else
+		vpninfo->ip6_fd = -1;
+
+	return tun_fd;
+}
+#else /* !__sun__ */
+
+/* MTU setting code for both Linux and BSD systems */
+static void ifreq_set_ifname(struct openconnect_info *vpninfo, struct ifreq *ifr)
+{
+	char *ifname = openconnect_utf8_to_legacy(vpninfo, vpninfo->ifname);
+	strncpy(ifr->ifr_name, ifname, sizeof(ifr->ifr_name) - 1);
+	if (ifname != vpninfo->ifname)
+		free(ifname);
+}
+
+static int set_tun_mtu(struct openconnect_info *vpninfo)
+{
+	struct ifreq ifr;
+	int net_fd;
+
+	net_fd = socket(PF_INET, SOCK_DGRAM, 0);
+	if (net_fd < 0) {
+		vpn_perror(vpninfo, _("open net"));
+		return -EINVAL;
+	}
+
+	memset(&ifr, 0, sizeof(ifr));
+	ifreq_set_ifname(vpninfo, &ifr);
+	ifr.ifr_mtu = vpninfo->ip_info.mtu;
+
+	if (ioctl(net_fd, SIOCSIFMTU, &ifr) < 0)
+		vpn_perror(vpninfo, _("SIOCSIFMTU"));
+
+	close(net_fd);
+	return 0;
+}
+
 #ifdef IFF_TUN /* Linux */
+intptr_t os_setup_tun(struct openconnect_info *vpninfo)
+{
+	int tun_fd = -1;
 	struct ifreq ifr;
 	int tunerr;
 
@@ -204,8 +237,7 @@ intptr_t os_setup_tun(struct openconnect_info *vpninfo)
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_flags = IFF_TUN | IFF_NO_PI;
 	if (vpninfo->ifname)
-		strncpy(ifr.ifr_name, vpninfo->ifname,
-			sizeof(ifr.ifr_name) - 1);
+		ifreq_set_ifname(vpninfo, &ifr);
 	if (ioctl(tun_fd, TUNSETIFF, (void *) &ifr) < 0) {
 		vpn_progress(vpninfo, PRG_ERR,
 			     _("TUNSETIFF failed: %s\n"),
@@ -215,52 +247,122 @@ intptr_t os_setup_tun(struct openconnect_info *vpninfo)
 	}
 	if (!vpninfo->ifname)
 		vpninfo->ifname = strdup(ifr.ifr_name);
-#elif defined(__sun__)
+
+	/* Ancient vpnc-scripts might not get this right */
+	set_tun_mtu(vpninfo);
+
+	return tun_fd;
+}
+#else /* BSD et al, including OS X */
+
+#ifdef SIOCIFCREATE
+static int bsd_open_tun(char *tun_name)
+{
+	int fd;
+	int s;
+	struct ifreq ifr;
+
+	fd = open(tun_name, O_RDWR);
+	if (fd == -1) {
+		s = socket(AF_INET, SOCK_DGRAM, 0);
+		if (s < 0)
+			return -1;
+
+		memset(&ifr, 0, sizeof(ifr));
+		strncpy(ifr.ifr_name, tun_name + 5, sizeof(ifr.ifr_name) - 1);
+		if (!ioctl(s, SIOCIFCREATE, &ifr))
+			fd = open(tun_name, O_RDWR);
+
+		close(s);
+	}
+	return fd;
+}
+#else
+#define bsd_open_tun(tun_name) open(tun_name, O_RDWR)
+#endif
+
+intptr_t os_setup_tun(struct openconnect_info *vpninfo)
+{
 	static char tun_name[80];
-	int unit_nr;
+	int unit_nr = 0;
+	int tun_fd = -1;
 
-	tun_fd = open("/dev/tun", O_RDWR);
-	if (tun_fd < 0) {
-		perror(_("open /dev/tun"));
-		return -EIO;
-	}
+#if defined(__APPLE__) && defined (HAVE_NET_UTUN_H)
+	/* OS X (since 10.6) can do this as well as the traditional
+	   BSD devices supported via tuntaposx. */
+	struct sockaddr_ctl sc;
+	struct ctl_info ci;
 
-	unit_nr = ioctl(tun_fd, TUNNEWPPA, -1);
-	if (unit_nr < 0) {
-		perror(_("Failed to create new tun"));
-		close(tun_fd);
-		return -EIO;
-	}
+	if (vpninfo->ifname) {
+		char *endp = NULL;
 
-	if (ioctl(tun_fd, I_SRDOPT, RMSGD) < 0) {
-		perror(_("Failed to put tun file descriptor into message-discard mode"));
-		close(tun_fd);
-		return -EIO;
-	}
+		if (!strncmp(vpninfo->ifname, "tun", 3))
+			goto do_bsdtun;
 
-	sprintf(tun_name, "tun%d", unit_nr);
-	vpninfo->ifname = strdup(tun_name);
-
-	vpninfo->ip_fd = link_proto(unit_nr, "/dev/udp", IFF_IPV4);
-	if (vpninfo->ip_fd < 0) {
-		close(tun_fd);
-		return -EIO;
-	}
-
-	if (vpninfo->ip_info.addr6) {
-		vpninfo->ip6_fd = link_proto(unit_nr, "/dev/udp6", IFF_IPV6);
-		if (vpninfo->ip6_fd < 0) {
-			close(tun_fd);
-			close(vpninfo->ip_fd);
-			vpninfo->ip_fd = -1;
-			return -EIO;
+		if (strncmp(vpninfo->ifname, "utun", 4) ||
+		    (unit_nr = strtol(vpninfo->ifname + 4, &endp, 10), !endp) ||
+		    (unit_nr && vpninfo->ifname[4] == '0') ||
+		    *endp) {
+			vpn_progress(vpninfo, PRG_ERR,
+				     _("Invalid interface name '%s'; must match 'utun%%d' or 'tun%%d'\n"),
+				     vpninfo->ifname);
+			return -EINVAL;
 		}
-	} else
-		vpninfo->ip6_fd = -1;
+	}
 
-#else /* BSD et al have /dev/tun$x devices */
-	static char tun_name[80];
-	int i;
+	tun_fd = socket(PF_SYSTEM, SOCK_DGRAM, SYSPROTO_CONTROL);
+	if (tun_fd < 0) {
+		vpn_progress(vpninfo, PRG_ERR,
+			     _("Failed to open SYSPROTO_CONTROL socket: %s\n"),
+			     strerror(errno));
+		goto utun_fail;
+	}
+
+	snprintf(ci.ctl_name, sizeof(ci.ctl_name), UTUN_CONTROL_NAME);
+
+	if (ioctl(tun_fd, CTLIOCGINFO, &ci) == -1) {
+		vpn_progress(vpninfo, PRG_ERR,
+			     _("Failed to query utun control id: %s\n"),
+			       strerror(errno));
+		close(tun_fd);
+		goto utun_fail;
+	}
+
+	sc.sc_id = ci.ctl_id;
+	sc.sc_len = sizeof(sc);
+	sc.sc_family = AF_SYSTEM;
+	sc.ss_sysaddr = AF_SYS_CONTROL;
+
+	do {
+		sc.sc_unit = unit_nr + 1;
+
+		if (!connect(tun_fd, (struct sockaddr * )&sc, sizeof(sc))) {
+			if (!vpninfo->ifname &&
+			    asprintf(&vpninfo->ifname, "utun%d", unit_nr) == -1) {
+				vpn_progress(vpninfo, PRG_ERR,
+					     _("Failed to allocate utun device name\n"));
+				close(tun_fd);
+				goto utun_fail;
+			}
+			return tun_fd;
+		}
+		unit_nr++;
+
+	} while (sc.sc_unit < 255 && !vpninfo->ifname);
+
+	vpn_progress(vpninfo, PRG_ERR,
+		     _("Failed to connect utun unit: %s\n"),
+		     strerror(errno));
+	close(tun_fd);
+
+ utun_fail:
+	/* If we were explicitly asked for a utun device, fail. Else try tuntaposx */
+	if (vpninfo->ifname)
+		return -EIO;
+
+	tun_fd = -1;
+ do_bsdtun:
+#endif /* __APPLE__ && HAVE_NET_UTUN_H */
 
 	if (vpninfo->ifname) {
 		char *endp = NULL;
@@ -300,8 +402,8 @@ intptr_t os_setup_tun(struct openconnect_info *vpninfo)
 	}
 #endif
 	if (tun_fd < 0) {
-		for (i = 0; i < 255; i++) {
-			sprintf(tun_name, "/dev/tun%d", i);
+		for (unit_nr = 0; unit_nr < 255; unit_nr++) {
+			sprintf(tun_name, "/dev/tun%d", unit_nr);
 			tun_fd = bsd_open_tun(tun_name);
 			if (tun_fd >= 0)
 				break;
@@ -315,20 +417,21 @@ intptr_t os_setup_tun(struct openconnect_info *vpninfo)
 		vpninfo->ifname = strdup(tun_name + 5);
 	}
 #ifdef TUNSIFHEAD
-	i = 1;
-	if (ioctl(tun_fd, TUNSIFHEAD, &i) < 0) {
-		perror(_("TUNSIFHEAD"));
+	unit_nr = 1;
+	if (ioctl(tun_fd, TUNSIFHEAD, &unit_nr) < 0) {
+		vpn_perror(vpninfo, _("TUNSIFHEAD"));
 		close(tun_fd);
 		return -EIO;
 	}
 #endif
-#endif /* BSD-style */
 
 	/* Ancient vpnc-scripts might not get this right */
 	set_tun_mtu(vpninfo);
 
 	return tun_fd;
 }
+#endif /* !IFF_TUN (i.e. BSD) */
+#endif /* !__sun__ */
 
 int openconnect_setup_tun_fd(struct openconnect_info *vpninfo, int tun_fd)
 {
@@ -347,15 +450,16 @@ int openconnect_setup_tun_fd(struct openconnect_info *vpninfo, int tun_fd)
 	return 0;
 }
 
-int openconnect_setup_tun_script(struct openconnect_info *vpninfo, char *tun_script)
+int openconnect_setup_tun_script(struct openconnect_info *vpninfo,
+				 const char *tun_script)
 {
 	pid_t child;
 	int fds[2];
 
-	vpninfo->vpnc_script = tun_script;
+	STRDUP(vpninfo->vpnc_script, tun_script);
 	vpninfo->script_tun = 1;
 
-	set_script_env(vpninfo);
+	prepare_script_env(vpninfo);
 	if (socketpair(AF_UNIX, SOCK_DGRAM, 0, fds)) {
 		vpn_progress(vpninfo, PRG_ERR, _("socketpair failed: %s\n"), strerror(errno));
 		return -EIO;
@@ -368,7 +472,8 @@ int openconnect_setup_tun_script(struct openconnect_info *vpninfo, char *tun_scr
 		if (setpgid(0, getpid()) < 0)
 			perror(_("setpgid"));
 		close(fds[0]);
-		setenv_int("VPNFD", fds[1]);
+		script_setenv_int(vpninfo, "VPNFD", fds[1]);
+		apply_script_env(vpninfo->script_env);
 		execl("/bin/sh", "/bin/sh", "-c", vpninfo->vpnc_script, NULL);
 		perror(_("execl"));
 		exit(1);
